#!/usr/bin/python

"""Linkers are in charge of creating symlinks for one of the tree branches.
"""

from __future__ import print_function

import logging
import re
import os
from glob import glob

from .config import config

logger = logging.getLogger(__name__)


def resolve_template(template, item):
    """Resolve template pattern, replacing placeholders by item values.
    """
    root_dir = config['link_root_dir'].as_filename()
    placeholders = re.findall(r'(\%\{?\w+\}?)', template)
    path = template
    for p in placeholders:
        path = path.replace(p, item[p.strip('%{}')])
    return os.path.join(root_dir, path)


def create_dir(link_dir):
    """A os.makedirs that doesn't throw.
    """
    if not os.path.exists(link_dir):
        os.makedirs(link_dir)


def dircmp(val, link_dir):
    """Return a positive distance if link_dir matches val or -1 if no match.
    """
    is_range = (link_dir[0] == '[') and (link_dir[-1] == ']'
                                         ) and link_dir.count('-') == 1
    link_dir = link_dir.strip('[]')
    if val.startswith(link_dir):
        return 0
    elif link_dir.endswith('+'):
        if val > link_dir[:-1]:
            return 1.0 / len(link_dir)
    elif is_range:
        _min, _max = link_dir.split('-')
        length = len(_min)
        if _min <= val[:length] <= _max:
            return 0
    return -1


def find_bucket(parent_dir, val):
    """Find a sub directory of parent_dir whose name is compatible with val.

       It occurs if val starts with folder name or if val belongs to folder
       name implied range.
    """
    candidates = glob(parent_dir + '/*/')
    bucket = None
    min_distance = 99
    for c in sorted([os.path.basename(c.strip(os.path.sep))
                    for c in candidates], key=lambda x: x.strip('[]').lower()):
        distance = dircmp(val.lower(), c.lower())
        if 0 <= distance <= min_distance:
            bucket = c
            min_distance = distance

    if not bucket:
        logger.warning("Found no folder defining a compatible "
            "range for '%s'" % val)
    return bucket


class Linker():
    def __init__(self, name):
        self.field = name
        self.config = config[name]
        self.config.add({'root': self.field,
                        'dirs': False,
                        'buckets': False})
        self.root = self.config['root'].get()
        self.dirs = self.config['dirs'].get()
        self.buckets = self.config['buckets'].get()
        default_link_format = '%s%s%s' % (
            ('%%%s' % self.field) if not self.dirs else '',
            '-' if (not self.dirs and self.field != 'title') else '',
            '%title' if self.field != 'title' else '')
        self.config.add({'link_format': default_link_format})
        self.link_format = self.config['link_format'].get()

    def flink(self, item):
        link_path = resolve_template(os.path.join(self.root,
                                     self.link_format), item)
        link_dir, link_name = os.path.split(link_path)
        last_dir = ''
        if self.dirs:
            last_dir = item[self.field]
        if self.buckets:
            bucket = find_bucket(link_dir, item[self.field])
            if bucket and bucket != last_dir:
                last_dir = os.path.join(bucket , last_dir)
        if last_dir:
            link_dir = os.path.join(link_dir, last_dir)
        create_dir(link_dir)
        dest = os.path.join(link_dir, link_name)
        if not os.path.lexists(dest):
            logger.debug('Linking %s' % dest)
            os.symlink(item['filename'], dest)
        else:
            logger.warning('%s already exist' % dest)




#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
    Flinck: Finger lickin good flicks linker.
"""

from __future__ import print_function

import logging
import os
import sys

import click

from . import confit
from . import brain

from .config import config, FIELDS
from .linker import Linker
from .__version__ import __version__

__author__ = 'Fabrice Laporte <kraymer@gmail.com>'
logger = logging.getLogger(__name__)


def set_logging(verbose):
    levels = {0: logging.WARNING, 1: logging.INFO, 2: logging.DEBUG}
    logger.setLevel(levels[verbose])
    ch = logging.StreamHandler()
    ch.setLevel(levels[verbose])
    ch.setFormatter(logging.Formatter('%(levelname)s: %(message)s'))
    logger.addHandler(ch)


def recursive_glob(treeroot):
    """Browse folders hierarchy and yield files with
    """
    min_size = config['file_min_size_mb'].get() * 1024 * 1024

    def is_video_file_candidate(path):
        return (path.endswith(tuple(config['file_extensions'].get())) and
                os.path.getsize(path) > min_size)

    if not os.path.isdir(treeroot) and is_video_file_candidate(treeroot):
        yield treeroot
    for base, _, files in os.walk(treeroot):
        for fpath in files:
            abs_path = os.path.join(base, fpath)
            if is_video_file_candidate(abs_path):
                yield abs_path


def flinck(media_src, *options):
    """Entry point to use flinck as a python module
       Command line options are passed as arguments, eg

           >>> flinck('movie.avi', '-b', 'genre', '-b', 'year')
    """
    flinck_cli([media_src] + list(options), standalone_mode=False)


@click.command(context_settings=dict(help_option_names=['-h', '--help']),
    help='Organize your movie collection using symbolic links.',
    epilog='Example: flinck -l ./ --by genre --by rating ~/Movies')
@click.argument('media_src', type=click.Path(exists=True), metavar='FILE|DIR')
@click.option('--link_dir', '-l', type=click.Path(exists=True),
    required=(not config['link_root_dir']),
    default=os.path.expanduser(config['link_root_dir'].get()),
    help='Links root directory')
@click.option('--by', '-b', multiple=True, type=click.Choice(sorted(FIELDS)),
    required=(not set(config.keys()).intersection(FIELDS)),
    help='Organize medias by...')
@click.option('-v', '--verbose', count=True)
@click.version_option(__version__)
def flinck_cli(media_src, link_dir, by, verbose):
    set_logging(verbose)
    if link_dir:
        config['link_root_dir'] = link_dir
    if not config['link_root_dir'] or not \
            os.path.exists(config['link_root_dir'].as_filename()):
        logger.error('Error: links root directory "%s" does not exist.' %
              config['link_root_dir'])
        exit(1)
    linkers = [Linker(field) for field in by]
    for fpath in recursive_glob(media_src):
        item = brain.search_filename(fpath, by)
        if item:
            for linker in linkers:
                linker.flink(item)
            logger.info('%s: done' % os.path.basename(fpath))
        else:
            logger.warning('%s: no result in Open Movie Database' %
                os.path.basename(fpath))


if __name__ == "__main__":
    flinck_cli()



from . import confit

FIELDS = ('country', 'director', 'decade', 'genre', 'rating', 'runtime',
    'title', 'year')

config = confit.LazyConfig('flinck', __name__)
config.add({
    'file_extensions': ['avi', 'm4v', 'mkv', 'mp4'],
    'file_min_size_mb': 20,
    'google_api_key': '',
    'link_root_dir': '',
    'verbose': False,
})



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016 Fabrice Laporte - kray.me
# The MIT License http://www.opensource.org/licenses/mit-license.php

import io
import os
import re

from setuptools import setup

def yield_sphinx_only_markup(lines):
    """http://stackoverflow.com/a/25900928/5181
       Cleans-up Sphinx-only constructs (ie from README.rst),
       so that *PyPi* can format it properly.
       To check for remaining errors, install ``sphinx`` and run::
       python setup.py --long-description | sed -file 'this_file.sed' | rst2html.py  --halt=warning

    """
    substs = [
        # Selected Sphinx-only Roles.
        (r':abbr:`([^`]+)`', r'\1'),
        (r':ref:`([^`]+)`', r'`\1`_'),
        (r':term:`([^`]+)`', r'**\1**'),
        (r':dfn:`([^`]+)`', r'**\1**'),
        (r':(samp|guilabel|menuselection):`([^`]+)`', r'``\2``'),

        # Sphinx-only roles:
        (r':(\w+):`([^`]*)`', r'\1(``\2``)'),

        # Sphinx-only Directives.
        (r'\.\. doctest', r'code-block'),
        (r'\.\. plot::', r'.. '),
        (r'\.\. seealso', r'info'),
        (r'\.\. glossary', r'rubric'),
        (r'\.\. figure::', r'.. '),

        # Other
        (r'\|version\|', r'x.x.x'),
    ]

    regex_subs = [(re.compile(regex, re.IGNORECASE), sub)
                  for (regex, sub) in substs]

    def clean_line(line):
        try:
            for (regex, sub) in regex_subs:
                line = regex.sub(sub, line)
        except Exception as ex:
            print(("ERROR: %s, (line(%s)" % (regex, sub)))
            raise ex
        return line

    for line in lines:
        yield clean_line(line)

with io.open('flinck/__version__.py', 'r') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]$', f.read(), re.MULTILINE).group(1)

readme_lines = open('README.rst').readlines()
setup(name='flinck',
    version=version,
    description='Sort your movies on filesystem using symlinks.',
    long_description=''.join(yield_sphinx_only_markup(readme_lines)),
    author='Fabrice Laporte',
    author_email='kraymer@gmail.com',
    url='https://github.com/KraYmer/flinck',
    license='MIT',
    platforms='ALL',
    packages=['flinck', ],

    entry_points={
        'console_scripts': [
            'flinck = flinck:flinck_cli',
        ],
    },
    install_requires=['click',
        'omdb',
        'pyyaml'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Environment :: Console',
        'Topic :: System :: Filesystems',
        'Topic :: Multimedia :: Video'
    ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""IMDB movie searcher.
"""

from __future__ import print_function

import logging
import os
import re
import sys
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

import omdb
import requests

from .config import config, FIELDS

# Regex to extract title and year. Should work as long as they are at the
# beginning and in that order.
FNAME_SPLIT_RE = '|'.join([r'\W%s(?:\W|$)' % x
                           for x in ('dvdrip', r'vost\w+', '1080p',
                                     '720p', 'multi',
                                     r'[\(\[]\D+[\)\]]',  # parentheses
                                     'bluray', 'x264', 'ac3',  # format
                                     r'b[dr]rip', 'xvid', 'divx', 'fansub',
                                     r'S\d+(E\d+)?',      # seasons
                                     'avi', 'mkv')])
CACHED_RESULTS = {}
logger = logging.getLogger(__name__)


def scrub(text, chars, new):
    """Replace chars.
    """
    for char in chars:
        if char in text:
            text = text.replace(char, new)
    return text.strip()


def google_search_by(title, year):
    engine_id = '009217259823014548361:0gf2jfpzpbm'
    url = (u'https://www.googleapis.com/customsearch/v1?key='
        '%s&cx=%s&q=%s+%s' % (config['google_api_key'], engine_id,
            quote(title), year))
    r = requests.get(url)
    if r.status_code == 200:
        json = r.json()
        if 'items' in json:
            return r.json()['items'][0]['link'].strip('/').split('/')[-1]


def format_item(item):
    """Tweak the string representation of the item field
    """
    for field in set(list(item) + list(FIELDS)):
        if item.get(field, None) == 'N/A':
            item[field] = 'Unknown'
    try:
        for field in ('country', 'genre'):
            item[field] = item[field].split(',')[0]
        item['director'] = item['director'].replace(', ', ' and ')
        item['runtime'] = re.findall(r'\d+', item['runtime']
                                     )[0].zfill(3) + ' min'
        item['decade'] = item['year'].strip(u'–')[:-1] + '0s'
        item['rating'] = item.pop('imdb_rating')
    except Exception:
        item[field] = 'Unknown'
    return item


def to_unicode(text):
    try:
        return unicode(text, sys.getfilesystemencoding(), errors="ignore")
    except NameError:
        pass  # Python3, no conversion needed
    return text


def search_by(title, year, fields, imdb_id=None):
    """Search movie infos using its title and year
    """
    if (title, year) in CACHED_RESULTS:
        item = CACHED_RESULTS[(title, year)]
        logger.debug('Get from cache: %s' % item)
    else:
        query = {'fullplot': False, 'tomatoes': False}
        if imdb_id:
            query['imdbid'] = imdb_id
        else:
            query['title'] = title
            query['year'] = year
        logger.debug('Query: %s' % query)
        item = omdb.get(**query)
        if item:
            item['title'] = to_unicode(title)  # force original title
            item = format_item(item)
            CACHED_RESULTS[(title, year)] = item
        elif not imdb_id and config['google_api_key']:
            imdb_id = google_search_by(title, year)
            if imdb_id:
                item = search_by(title, year, fields, imdb_id)
    return item

def search_filename(fname, fields):
    """Extract movie title/date from filename and return dict with movies infos
    """
    path_tokens = os.path.normpath(fname).split(os.sep)
    candidate = path_tokens[-1]
    res = re.split(FNAME_SPLIT_RE, candidate,
                   flags=re.I | re.U)[0].strip()
    res = scrub(res, '[({])}', ' ')
    res = ' '.join([x for x in re.split(r'[\s\._]', res, flags=re.U) if x])
    years = re.findall(r'((?:19|20)\d\d)', res)
    if years:
        toks = re.split(r'(%s)' % years[-1], res)
    else:
        toks = [res]
    title = toks[0].strip()
    year = toks[1] if len(toks) > 1 else None
    item = search_by(title, year, fields)
    if item:
        item['filename'] = fname
        return item
